# -*- coding: utf-8 -*-
# Copyright (c) 2000-2004 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
""" schedule project using Constraint Solving Programing """

__revision__ = "$Id: csp.py,v 1.2 2005-09-07 23:51:01 nico Exp $"

from mx.DateTime import oneDay, oneHour, today
from logilab.common.compat import set
import projman.lib.constants as CST
from gcsp import ProjmanProblem, solve, constraint_types as GCSP_CST, load_types

GCSPMAP = {}
for t in CST.TASK_CONSTRAINTS:
    name = t.upper().replace("-","_")
    GCSPMAP[t] = getattr(GCSP_CST, name)

_VERBOSE=1

class CSPScheduler:
    """
    schedule a project using constraint programming
    
    variables = leaf tasks of the project
    values = (timeslotsset, resource)
    """
    def __init__(self, project):
        self.project = project
        self.start_date = None
        self.calc_project_length()
        #self.first_tasks = None
        self.real_tasks = {}  # task_id -> (task_number,duration,list_of_resources)
        self.constraints = {} # Constraint Type -> list of task pairs
        self.task_ranges = {} # task_id -> date range or (None,None)
        self.resources = set()   # res_id -> res_number
        for leaf in project.root_task.leaves():
            self._process_node(leaf)
        #self.add_priorities_as_constraints()

    def calc_project_length(self):
        """Computes start_date and max_duration (duration of the project)"""
        max_duration = self.project.root_task.maximum_duration()
        begins, ends = [], []
        for leaf in self.project.root_task.leaves():
            for c_type, date, priority in leaf.get_date_constraints():
                if c_type in (CST.BEGIN_AT_DATE, CST.BEGIN_AFTER_DATE) and \
                            self.project.priority >= int(priority):
                    begins.append(date)
                elif c_type in (CST.END_AT_DATE, CST.END_BEFORE_DATE) and \
                            self.project.priority >= int(priority):
                    ends.append(date)
        if not self.start_date:
            # We take the earliest date constraint as a start date
            if begins:
                self.start_date = min(begins) 
            elif ends:
                self.start_date = min(ends)
            else:
                self.start_date = today()
        # find the first working day
        d = self.start_date
        self.first_day = 0
        while True:
            # this really should be the list of resources working
            # on those task that *could* begin at the start of the project 
            for res_id in self.project.get_resources():
                res = self.project.get_resource(res_id)
                if res.is_available( d ):
                    break
            else:
                d = d + 1
                self.first_day += 1
                continue
            break
        if ends and begins:
            other_length = (max(ends) - min(begins)).days +1
        else:
            other_length = 0
        self.max_duration = max(max_duration, other_length)

    def _process_node(self, node):
        max_duration = int(self.max_duration*2)

        _, _, _, task_resources = self.real_tasks.setdefault( node.id,
                                                           [len(self.real_tasks),
                                                            node.load_type,
                                                            node.duration, []] )
        rnge = self.task_ranges.setdefault( node.id, [None,None] )

        # collect task constraints
        for constraint_type, task_id, priority in node.get_task_constraints():
            if self.project.priority >= int(priority):
                for leaf in node.get_task(task_id).leaves():
                    lst = self.constraints.setdefault(constraint_type, set())
                    lst.add( (node.id, leaf.id)  )
        # collect date constraints
        tab_rnge0 = [0]
        tab_rnge1 = [max_duration]
        for c_type, date, priority in node.get_date_constraints():
            if self.project.priority >= int(priority):
                days = (date-self.start_date).days
                if days<0:
                    raise Exception('WTF?')
                if c_type == CST.BEGIN_AFTER_DATE :
                    tab_rnge0.append(days)
                elif c_type == CST.END_BEFORE_DATE:
                    tab_rnge1.append(days+1)
        rnge[0] = max(tab_rnge0)
        rnge[1] = min(tab_rnge1)
        if _VERBOSE>2:
            print node.id, 'range=', rnge
        # collect resources
            # 
            # on utilise task.set_resources pour obtenir l ensemble des
            # resources disponibles pour une tache
            #
        # -> using new projman definition
        if node.TYPE != 'milestone' and node.get_resource_constraints()== set():
            # collect resources in root_trask
            for r_type, r_id in self.project.root_task.get_resource_constraints():
                if self.project.priority >= int(priority):
                    self.resources.add( r_id )
                    task_resources.append( r_id )
            self.project.get_resources_from_task_type(node)
            #trouver les resources correspondantes
            task_type = node.get_task_type()
            if _VERBOSE>1:
                print "Resource", r_id
        # -> using old projman definition
        else:
            for r_type, r_id in node.get_resource_constraints():
                if self.project.priority >= int(priority):
                    if _VERBOSE>1:
                        print "Resource", r_type, r_id
                    self.resources.add( r_id )
                    task_resources.append( r_id )
                    node.set_resources.append(r_id)

    def add_priorities_as_constraints(self):
        """
        transform priorities as BEGIN_AFTER_END constraints
        """
        return # XXXX FIXME
        lbp = {}
        for task in self.project.root_task.leaves():
            lbp.setdefault(task.priority, []).append(task)
        priorities = lbp.keys()
        priorities.sort()
        for (low, high) in zip(priorities[:-1], priorities[1:]):
            for low_leaf in lbp[low]:
                for high_leaf in lbp[high]:
                    self.constraints.add(fi.StartsAfterEnd(high_leaf.id,
                                                           low_leaf.id))

    def activity_usage_counter_by_task(self, activities, tid):
        """
        count number of days already affected to the task(tid)
        """
        usage = 0
        for element in activities:
            if tid == element[3]:
                usage = usage + element[4]
        return usage

    def schedule(self, verbose=0, time=400000, **kw):
        """
        Update the project's schedule
        Return list of errors occured during schedule
        """
        print "\nscheduling ..."
        _VERBOSE = verbose
        # check the tasks (duration is not 0)
        for leaf in self.project.root_task.leaves():
            leaf.check_duration()
        self.project.get_factor()
        factor = self.project.factor
        self.max_duration = int( self.max_duration * 2 )
        if _VERBOSE>1:
            print "Tasks", len(self.real_tasks)
            print "Res", len(self.resources)
            print "Dur", self.max_duration
            print "Factor", factor
        pb = ProjmanProblem( int(self.max_duration * factor ) )
        pb.set_first_day( self.first_day * factor) 
        real_tasks_items = self.real_tasks.items()
        real_tasks_items.sort( key = lambda x:x[1][0] )
        dt = []
        if _VERBOSE>1:
            print "occupation"
            print "----------"
        resources_map = {}
        for res_id in self.resources:
            sched = []
            res = self.project.get_resource( res_id )
            res_num = pb.add_worker( res_id )
            resources_map[res_id] = res_num
            #gestion calendrier jours feries et we
            for d in range(int(self.max_duration)):
                dt = self.start_date + d
                if not res.is_available( dt ):
                    for i in range(factor):
                        pb.add_not_working_day( res_num, d*factor + i )
                        sched.append("x")
                else:
                    for i in range(factor):
                        sched.append(".")
            if _VERBOSE > 1:
                print "%02d" % res_num, "".join(sched)
        pseudo_tasks = []
        i = 0

        for tid, (num, _type, duration, resources) in real_tasks_items:

            task = self.project.get_task(tid)
            resources = task.set_resources # set resources according to new definition
            duration_ = duration * factor
            if (duration * factor) % 1 > 0 :
                duration_ = duration * factor - ((duration * factor) % 1) + 1
                real_tasks_items[i][1][2] = duration_ / factor
            task_num = pb.add_task( tid, _type, int(duration_), bool(task.can_interrupt[0]) )
            low, high = self.task_ranges[tid]
            if _VERBOSE>1:
                print "Task %2d = #%.2f [%4s,%4s] = '%20s'" % ((task_num,duration,low,high,tid,))
            if low is None:
                low = 0
            else:
                low *= factor
            if high is None:
                high = self.max_duration * factor
            else:
                high *= factor
            pb.set_task_range( task_num, int(low), int(high), 0, 0 ) # XXX: cmp_type unused
            if _type == load_types.TASK_MILESTONE:
                continue
            for res_id in sorted(resources):
                res_num = resources_map[res_id]
                pseudo_id = pb.add_resource_to_task( task_num, res_num ) 
                pseudo_tasks.append( (pseudo_id, tid, res_id) )

        # register constraints
        for type, pairs in self.constraints.items():
            for t1, t2 in pairs:
                n1, _, _, _ = self.real_tasks[t1]
                n2, _, _, _ = self.real_tasks[t2]
                pb.add_task_constraint( GCSPMAP[type], n1, n2 )
                if _VERBOSE>1:
                    print "%s %s(%s), %s(%s)" %(type, t1, n1, t2, n2)

        pb.set_convexity( True )
        pb.set_time(time)
        pb.set_verbosity( _VERBOSE )
        pb.set_max_nb_solutions(4000)
        solve( pb )
        
        self.project.nb_solution = pb.get_number_of_solutions()
        if self.project.nb_solution==0:
            return []
        if (_VERBOSE>2):
            self.compare_solutions( pb )
        SOL = pb.get_solution( self.project.nb_solution-1 )
        duration = SOL.get_duration()
        ntasks = SOL.get_ntasks()
        tasks_days = [ [ day / factor for day in range(duration) if SOL.isworking( task, day ) ] for task in range(ntasks) ]
        #calendar gere l'utilisation d'une resource pour un jour afin
        # d'organiser les heures de travail si factorized_days > 1
        calendar = []# attention si le calendrier est fonction de chaque resources
                     # ca ne marche plus
                     # ca ne marche pas non plus si la resource n est pas utilisee
                     # sur une autre tache avant ...
        for i in range(duration/factor):
            calendar.append([])
            for j in range(len(resources_map)):
                calendar[i].append(0) 

        activities = []
        for pid, days in enumerate( tasks_days ):
            num, tid, res_id = pseudo_tasks[pid]
            time_table = oneHour * CST.HOURS_PER_DAY / factor
            for i, d in enumerate(days):
                #print "debug"
                #print "day:", d, i
                decalage = calendar[d][resources_map[res_id]]* time_table
                if decalage == 0:# and i+1 < len(days):
                    if i+1 < len(days) and days[i+1] != d and factor == 2:
                        decalage += CST.HOURS_PER_DAY * oneHour /factor #ok
                        calendar[d][resources_map[res_id]] += 2
                    elif i+1 < len(days) and factor == 4 and days[i+1] != d:
                        decalage += CST.HOURS_PER_DAY * oneHour /(factor/3.)
                        calendar[d][resources_map[res_id]] += 4
                    elif i+2 < len(days) and factor == 4 and days[i+2] != d:
                        decalage += CST.HOURS_PER_DAY * oneHour /(factor/2.)
                        calendar[d][resources_map[res_id]] += 3
                    elif i+3 < len(days) and factor == 4 and days[i+3] != d:
                        decalage += CST.HOURS_PER_DAY * oneHour /float(factor)
                        calendar[d][resources_map[res_id]] += 2
                    else:
                        calendar[d][resources_map[res_id]] += 1
                else:
                    calendar[d][resources_map[res_id]] += 1
                date = self.start_date + d + CST.HOURS_PER_DAY * oneHour + decalage
                if date.hour == 0:
                    date += CST.HOURS_PER_DAY * oneHour
                elif date.hour > 17:
                    date += CST.HOURS_PER_DAY * oneHour
                    date += oneDay
                elif date.hour >= 12:
                    date += oneHour
                if calendar[d][resources_map[res_id]]> factor:
                    raise Exception("found non valid solution")
                delta = self.real_tasks[tid][2] * factor - \
                    self.activity_usage_counter_by_task( activities, tid )* factor 
                # duree (initiale) tache * factor - nb de jours 
                #                        deja ecoules pr cette tache
                if delta > 1.0 / factor or delta <= 0:
                    usage = 1.0 / factor
                else:
                    usage = delta
                activities.append((date, date + time_table, res_id, 
                                tid, max(usage,1./factor)))
        if _VERBOSE > 0:
            print "\nactivites :"
            for (db, de, res_id, tid, dur) in activities:
                print "\tdu", db,"au", de, res_id, tid, dur
        milestone = 0
        nmilestones = SOL.get_nmilestones()
        for tid, (num, _type, duration, resources) in self.real_tasks.items():
            if duration!=0:
                continue
            if milestone>=nmilestones:
                break
            d = SOL.get_milestone( milestone )
            date = self.start_date + CST.HOURS_PER_DAY * oneHour + int(d / factor)
            if (_VERBOSE>=2):
                print "MILESTONE", tid, date
            self.project.milestones[tid] = date
            milestone += 1

        self.project.add_schedule(activities)
        return []

    def read_sol( self, SOL ):
        duration = SOL.get_duration()
        ntasks = SOL.get_ntasks()
        tasks_days = [ [ day for day in range(duration) if SOL.isworking( task, day ) ] for task in range(ntasks) ]
        return tasks_days

    def compare_solutions( self, pb ):
        N = pb.get_number_of_solutions()
        if N==0:
            return
        SOL0 = self.read_sol( pb.get_solution( 0 ) )
        for i in range(1,N):
            SOL1 = self.read_sol( pb.get_solution( i ) )
            for id, (task0,task1) in enumerate( zip( SOL0, SOL1 ) ):
                if task0!=task1:
                    print id, task0
                    print id, task1
            print
            SOL0 = SOL1

def solution_cost(solution):
    """cost function
    
    we try to minimize the end date of the project, so the cost of a
    solution maybe represented by the end date of the last task
    """
    end_dates = [var[0].get_end() for var in solution.values()]
    return max(end_dates)





# -*- coding: iso-8859-1 -*-
# Copyright (c) 2004-2005 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
Projman - (c)2000-2005 LOGILAB <contact@logilab.fr> - All rights reserved.

Home: http://www.logilab.org/projects/projman

Manipulate a xml project description.

This code is released under the GNU Public Licence v2. See www.gnu.org.
"""

__revision__ = "$Id: resource.py,v 1.4 2005-09-06 17:07:00 nico Exp $"

from mx.DateTime import Time
from logilab.common.deprecation import deprecated_function

from logilab.common.tree import VNode 
from projman.lib.calendar import Calendar

class Resource(VNode):
    """A Resource instance is able to perform Tasks.

    Attributes:
      - type: resource classifier (e.g. 'engineer',
        'manager', 'computer'...)
      - calendar a Calendar instance to indicating when the resource is
         (not) available
      - hourly_rate cost of one hour of work by the resource stored as
         [rate, 'unit']
     """

    TYPE = 'resource'

    def __init__(self, r_id, name, type=None, calendar=None,
                 hourly_rate=0.0, unit='euros'):
        VNode.__init__(self, r_id)
        self.name = name
        self.type = type
        self.calendar = calendar
        self.hourly_rate = [hourly_rate, unit]
        self.id_role = []

    def get_default_wt_in_hours(self):
        """return number of working hours for this resource in a default day"""
        if self.calendar:
            cal = self.get_node_by_id(self.calendar)
            return cal.get_worktime(cal.default_day_type).hours
        else:
            return 8

    def is_available(self, datetime): 
        """ 
        tell if the resource may work on a given day
        """
        if self.calendar:
            c_init = self.get_node_by_id(self.calendar)
            if c_init.availability(datetime):
                return True
            else:
                return False
        else:
            return True
    work_on = deprecated_function(is_available)
    
    def get_worktime(self, datetime):
        """
        return the number of seconds of availability on a given day
        """
        if not self.is_available(datetime):
            return Time(0)
        if self.calendar:
            cal = self.get_node_by_id(self.calendar)
            daytype = cal.get_daytype(datetime)
            return cal.get_worktime(daytype)
        else:
            # if no calendar use default value of 8 hours work
            return Time(8)

class ResourcesSet(VNode):

    TYPE = 'resourcesset'

    def __init__(self, r_id):
        VNode.__init__(self, r_id)

    add_resource = VNode.append
    get_resource = VNode.get_node_by_id
    add_calendar = VNode.append

    def get_calendar(self, tt_id):
        """
        return the time table projman object with id
        """
        calendar = self.get_node_by_id(tt_id)
        assert isinstance(calendar, Calendar)
        return calendar

    def get_resources(self):
        """
        return the list of ids of all resources available in this resources set
        """
        return [r.id for r in self.flatten() if isinstance(r, Resource)]




# -*- coding: iso-8859-1 -*-
"""
unit tests for module logilab.projman.lib.ResourceSet

Projman - (c)2000-2002 LOGILAB <contact@logilab.fr> - All rights reserved.
Home: http://www.logilab.org/projman

Manipulate a xml project description.
This code is released under the GNU Public Licence v2. See www.gnu.org.

"""

from logilab.common import testlib

from projman.lib import Resource, Calendar, ResourcesSet
from mx.DateTime import DateTime, Time
    
class ResourceTest(testlib.TestCase):
    """
    Resource represents 
    """
    def setUp(self):
        """ called before each test from this class """
        # set up resources
        self.r1 = Resource('r_1', 'Resource 1')
        self.r1.calendar = 'c_1'
        self.r2 = Resource('unicode', u'élise')
        self.r2.calendar = 'c_2'
        # set of dates
        self.date_last_week = DateTime(2004, 10, 1)
        self.date_today = DateTime(2004, 10, 7)
        self.date_tomorrow = DateTime(2004, 10, 8)
        self.date_next_week = DateTime(2004, 10, 13)
        self.date_next_score = DateTime(2004, 10, 26)
        # set up calendar 1      
        self.c1 = Calendar('c_1', 'Defaut')
        type_working_days_c1 = {0:['Working', [(Time(8., 0.), Time(12., 0.)),
                                               (Time(13., 0.), Time(17., 0.))]],
                                1:['HalfDay', [(Time(9., 0., 0.), Time(15., 0., 0.))]]}
        type_nonworking_days_c1 = {0:'Use base',
                                   1:'Nonworking'} 
        self.c1.type_working_days = type_working_days_c1
        self.c1.type_nonworking_days = type_nonworking_days_c1
        self.c1.add_timeperiod(self.date_last_week, self.date_last_week, 'Nonworking')
        self.c1.add_timeperiod(self.date_today, self.date_today, 'Working')
        self.c1.add_timeperiod(self.date_tomorrow, self.date_next_week, 'HalfDay')
        self.c1.weekday['mon'] = 'Working'
        self.c1.weekday['tue'] = 'Working'
        self.c1.weekday['wed'] = 'Working'
        self.c1.weekday['thu'] = 'Working'
        self.c1.weekday['fri'] = 'Working'
        self.c1.weekday['sat'] = 'Nonworking'
        self.c1.weekday['sun'] = 'Nonworking'
        # set up calendar 2        
        type_working_days_c2 = {}
        type_nonworking_days_c2 = {0:'Use base',
                                   1:'Nonworking'}
        self.c2 = Calendar('c_2', u'Année')
        self.c2.type_working_days = type_working_days_c2
        self.c2.type_nonworking_days = type_nonworking_days_c2
        self.c2.add_timeperiod(self.date_next_week, self.date_next_score, 'Nonworking')
        self.c2.weekday['mon'] = 'Use base'
        self.c2.weekday['tue'] = 'Use base'
        self.c2.weekday['wed'] = 'Use base'
        self.c2.weekday['thu'] = 'Use base'
        self.c2.weekday['fri'] = 'Use base'
        self.c2.weekday['sat'] = 'Use base'
        self.c2.weekday['sun'] = 'Use base'
        # build tree
        self.c1.append(self.c2)
        self.rss = ResourcesSet('all_resources')
        self.rss.add_resource(self.r1)
        self.rss.add_resource(self.r2)
        self.rss.add_calendar(self.c1)
        
    def test_get_resource(self):
        """
        """
        self.assertEquals(self.rss.get_resource('r_1'), self.r1)
        self.assertEquals(self.rss.get_resource('unicode'), self.r2)

    def test_get_resources(self):
        """
        """
        self.assertEquals(self.rss.get_resources(), ['r_1', 'unicode'])

    def test_get_calendar(self):
        """
        """
        self.assertEquals(self.rss.get_calendar('c_1'), self.c1)
        self.assertEquals(self.rss.get_calendar('c_2'), self.c2)
    
if __name__ == '__main__':
    testlib.unittest_main()



#!/usr/bin/env python
# pylint: disable-msg=W0404
# Copyright (c) 2003 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
""" Generic Setup script, takes package info from __pkginfo__.py file """

from __future__ import nested_scopes

import os
import sys
import shutil
from distutils.core import setup
from os.path import isdir, exists, join, walk

# import required features
from __pkginfo__ import modname, version, license, short_desc, long_desc, \
     web, author, author_email
# import optional features
import __pkginfo__
distname = getattr( __pkginfo__, 'distname', modname )
scripts = getattr( __pkginfo__, 'scripts', [] )
data_files = getattr( __pkginfo__, 'data_files', None )
subpackage_of = getattr( __pkginfo__, 'subpackage_of', None )
include_dirs = getattr( __pkginfo__, 'include_dirs', [] )
ext_modules = getattr( __pkginfo__, 'ext_modules', None )

BASE_BLACKLIST = ('CVS', 'debian', 'dist', 'build', '__buildlog')
IGNORED_EXTENSIONS = ('.pyc', '.pyo', '.elc')
    
def ensure_scripts(linuxScripts):
    """
    Creates the proper script names required for each platform
    (taken from 4Suite)
    """
    from distutils import util
    _scripts = linuxScripts
    if util.get_platform()[:3] == 'win':
        _scripts = [script + '.bat' for script in _scripts]
    return _scripts


def get_packages(directory, prefix):
    """return a list of subpackages for the given directory
    """
    result = []
    for package in os.listdir(directory):
        absfile = join(directory, package)
        if isdir(absfile):
            if exists(join(absfile, '__init__.py')) or \
                   package in ('test', 'tests'):
                if prefix:
                    result.append('%s.%s' % (prefix, package))
                else:
                    result.append(package)
                result += get_packages(absfile, result[-1])
    return result

def export(from_dir, to_dir,
           blacklist=BASE_BLACKLIST,
           ignore_ext=IGNORED_EXTENSIONS):
    """make a mirror of from_dir in to_dir, omitting directories and files
    listed in the black list
    """
    def make_mirror(arg, directory, fnames):
        """walk handler"""
        for norecurs in blacklist:
            try:
                fnames.remove(norecurs)
            except ValueError:
                pass
        for filename in fnames:
            # don't include binary files
            if filename[-4:] in ignore_ext:
                continue
            if filename[-1] == '~':
                continue
            src = '%s/%s' % (directory, filename)
            dest = to_dir + src[len(from_dir):]
            print >> sys.stderr, src, '->', dest
            if os.path.isdir(src):
                if not exists(dest):
                    os.mkdir(dest)
            else:
                if exists(dest):
                    os.remove(dest)
                shutil.copy2(src, dest)
    try:
        os.mkdir(to_dir)
    except OSError, ex:
        # file exists ?
        import errno
        if ex.errno != errno.EEXIST:
            raise
    walk(from_dir, make_mirror, None)


EMPTY_FILE = '"""generated file, don\'t modify or your data will be lost"""\n'

def install(**kwargs):
    """setup entry point"""
    if subpackage_of:
        package = subpackage_of + '.' + modname
        kwargs['package_dir'] = {package : '.'}
        packages = [package] + get_packages(os.getcwd(), package)
    else:
        kwargs['package_dir'] = {modname : '.'}
        packages = [modname] + get_packages(os.getcwd(), modname)
    kwargs['packages'] = packages
    dist = setup(name = distname,
                 version = version,
                 license =license,
                 description = short_desc,
                 long_description = long_desc,
                 author = author,
                 author_email = author_email,
                 url = web,
                 scripts = ensure_scripts(scripts),
                 data_files=data_files,
                 ext_modules=ext_modules,
                 **kwargs
                 )
    
    if dist.have_run.get('install_lib'):
        _install = dist.get_command_obj('install_lib')
        if subpackage_of:
            # create Products.__init__.py if needed
            product_init = join(_install.install_dir, subpackage_of,
                                '__init__.py')
            if not exists(product_init):
                stream = open(product_init, 'w')
                stream.write(EMPTY_FILE)
                stream.close()
        
        # manually install included directories if any
        if include_dirs:
            if subpackage_of:
                base = join(subpackage_of, modname)
            else:
                base = modname
            for directory in include_dirs:
                dest = join(_install.install_dir, base, directory)
                export(directory, dest)
    return dist
            
if __name__ == '__main__' :
    install()



# Copyright (c) 2000-2005 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""
Projman - (c)2005 Logilab - All rights reserved.
"""

import os.path as osp
import tempfile, shutil

from logilab.common import testlib
from projman.renderers import PILHandler, GanttRenderer, ResourcesRenderer

from projman.test import DATADIR
from projman.readers import ProjectXMLReader

CONFIG = testlib.AttrObject(del_ended=False, del_empty=False,
                            timestep=1, depth=0, selected_resource=None,
                            view_begin=None, view_end=None,
                            project_file=osp.join(DATADIR,"full_scheduled_projman.xml"),
                            task_root=None,
                            )

if __name__ == '__main__':
    testlib.unittest_main()



# -*- coding: iso-8859-1 -*-
"""
unit tests for module projman.lib.projman_reader.py
Projman - (c)2000-2006 LOGILAB <contact@logilab.fr> - All rights reserved.

Home: http://www.logilab.org/projman
Manipulate a xml project description.

This code is released under the GNU Public Licence v2. See www.gnu.org.

"""

import os.path as osp
from mx.DateTime import Time

from logilab.common.testlib import TestCase, unittest_main

from projman.readers import ProjectXMLReader
from projman.lib._exceptions import MalformedProjectFile
from projman.test import DATADIR

class DummyConfig:
    def __init__(self):
        self.task_root = None

class TaskXMLReaderTest(TestCase):

    def setUp(self):
        self.reader = ProjectXMLReader(None)
        self.root = self.reader.read_tasks(osp.join(DATADIR, 'multiline_tasked_project.xml'))
        #self.reader.project.root_task = self.root

    def test_multiline_project_label(self):
        expected_title = "Simplest Project with a multiline label, gosh can you believe it"
        self.assertEquals(expected_title, self.root.title)
        self.assertEquals(len(self.root.children), 3)

    def test_multiline_task_desc(self):
        task = self.root.children[0]
        expected_desc = u"Réunions de début et de fin de tranche, réunions\n      hebdomadaires, <emphasis>comptes-rendus</emphasis>, etc."
        self.assertEquals(expected_desc, task.description)

    def test_multiline_task_duration(self):
        task = self.root.children[0]
        self.assertEquals(25, task.duration)
        task = self.root.children[1]
        self.assertEquals(10.6, task.duration)
        task = self.root.children[2]
        self.assertEquals(0, task.duration)

    def test_multiline_task_progress(self):
        task = self.root.children[0]
        self.assertEquals(0, task.progress)


class TaskXMLReaderVirtualRootTest(TestCase):
    def setUp(self):
        self.reader = ProjectXMLReader(None, task_root='t1_1')
        self.root = self.reader.read_tasks(osp.join(DATADIR, 'multiline_tasked_project.xml'))

    def test_virtual_root(self):
        task = self.root
        expected_title = "Suivi de projet"
        self.assertEquals(expected_title, task.title)
        self.assertEquals(len(task.children), 0)
        expected_desc = u"Réunions de début et de fin de tranche, réunions\n      hebdomadaires, <emphasis>comptes-rendus</emphasis>, etc."
        self.assertEquals(expected_desc, task.description)
        self.assertEquals(25, task.duration)
        self.assertEquals(0, task.progress)

class ResourcesXMLReaderTest(TestCase):
    def setUp(self):
        self.reader = ProjectXMLReader(None)
        self.resources = self.reader.read_resources(osp.join(DATADIR, 'three_resourced_list.xml'))

    def test_number_of_resources(self):
        self.assertEquals(len(self.resources.children), 4)
        for res in self.resources.children[:-1]:
            self.assertEquals(res.TYPE,'resource')
        self.assertEquals(self.resources.children[-1].TYPE, 'calendar')

    def test_resource_content(self):
        res = self.resources.children[0]
        self.assertEquals(res.name, "Emmanuel Breton")
        self.assertEquals(res.hourly_rate, [80, "euros"])
        self.assertEquals(res.calendar, 'typic_cal')

    def test_calendar_content(self):
        cal = self.resources.children[-1]
        self.assertEquals(cal.name, "Calendrier Francais")
        self.assertEquals(cal.weekday, {'sat': 'non_working', 'sun':'non_working'} )
        self.assertEquals(cal.national_days,
                          [(1,1), (5,1), (5,8), (7,14),
                           (8,15), (11,1), (11,11), (12,25)])
        self.assertEquals(cal.start_on, None)
        self.assertEquals(cal.stop_on, None)
        self.assertEquals(cal.day_types,
                          {'working': [u'Standard work day', [(Time(8), Time(12)),
                                                             (Time(13), Time(17,24))]],
                           'non_working': [u'Week-end day', []],
                           'holiday': [u'Day Off',[]],})
        self.assertEquals(cal.default_day_type, 'working')
        dates = [("2002-12-31","2002-12-26"),
                 ("2003-03-14","2003-03-10"),
                 ("2003-08-18","2003-08-14"),
                 ("2004-05-21","2004-05-20")]
        for (expected_end, expected_start), (start, end, working) in zip(dates, cal.timeperiods):
            start = str(start).split()[0]
            end = str(end).split()[0]
            self.assertEquals(start, expected_start)
            self.assertEquals(end, expected_end)
            self.assertEquals(working, 'holiday')



class ErrorXMLReaderTest(TestCase):
    def setUp(self):
        self.reader = ProjectXMLReader(None)
    
    def test_error_project(self):
        self.assertRaises(Exception, self.reader.read_resources, osp.join(DATADIR, 'error_project.xml'))

    def test_error_doubletask(self):
        root = self.reader.read_tasks(osp.join(DATADIR, 'error_doubletask.xml'))
        self.assertEquals(root.check_consistency(), ['Duplicate task id: double_t1_1'])

      
    def test_error_dtd_project(self):
        self.assertRaises(MalformedProjectFile, self.reader.read_tasks, osp.join(DATADIR, 'error_dtd_project.xml'))
        reader = ProjectXMLReader(osp.join(DATADIR, 'error_dtd_projman.xml'))
        self.assertRaises(MalformedProjectFile, reader.read)


    def test_error_dtd_project_multi(self):
        try:
            self.reader.read_tasks(osp.join(DATADIR, 'multi_error_dtd_project.xml'))
        except MalformedProjectFile, ex:
            # more than one line of errors
            self.assertEquals(len(str(ex).split('\n')), 4)

if __name__ == '__main__':
    unittest_main()




import os
import gtk
import gobject
from projman.lib.constants import LOAD_TYPE_MAP, TASK_CONSTRAINTS
from projman.lib.task import Task
from projman.readers.base_reader import MODEL_FACTORY
import gtksourceview2

LANGUAGES = gtksourceview2.language_manager_get_default().get_language_ids()
LANGUAGES.sort()
assert "docbook" in LANGUAGES
XMLLANG = gtksourceview2.language_manager_get_default().get_language("docbook")
print LANGUAGES

class TaskEditor(gobject.GObject):

    __gsignals__ = {'task-changed': (gobject.SIGNAL_RUN_FIRST,
                                     gobject.TYPE_NONE,
                                     (gobject.TYPE_PYOBJECT,) ),
                    }

    def __init__(self, app):
        gobject.GObject.__init__(self)
        self.app = app
        self.w = app.ui.get_widget
        self.current_task = None
        self.current_task_path = None
        self.current_activity_path = None
        self.task_popup = None
        self.setup_ui()
        app.ui.signal_autoconnect(self)
        app.connect("project-changed", self.on_project_changed )
        self.w("spinbutton_duration").get_adjustment().set_all(0,0,100000,0.1,1,1)

    def setup_ui(self):
        self.constraints_type_model = gtk.ListStore(gobject.TYPE_STRING)
        for v in TASK_CONSTRAINTS:
            self.constraints_type_model.append( (v,) )
        self.setup_task_tree()
        self.setup_activities_tree()
        self.setup_constraints_tree()
        self.setup_resources_tree()

    def setup_project_files_path(self):
        self.w("entry_project_tasks_file").set_text(self.app.files['tasks'])
        self.w("entry_project_activites_file").set_text(self.app.files['activities'])
        self.w("entry_project_resources_file").set_text(self.app.files['resources'])
        self.w("entry_project_schedule_file").set_text(self.app.files['schedule'])
        self.w("window_main").set_title("Projman - "+str(self.app.project_file))

    def build_task_tree_popup(self, task_path, del_task=True):
        task_popup = gtk.Menu()
        task = self.get_task_from_path(task_path)

        if not self.app.project is None:
            if self.app.project.root_task and task_path and task.TYPE != "milestone":
                add_item = gtk.MenuItem("Add task")
                add_item.connect("activate", self.popup_add_task, task_path )
                task_popup.attach(add_item, 0, 1, 0, 1 )
                add_item = gtk.MenuItem("Add milestone")
                add_item.connect("activate", self.popup_add_milestone, task_path )
                task_popup.attach(add_item, 0, 1, 1, 2 )

        if del_task and task_path is not None:
            if task.TYPE != "milestone":
                del_item = gtk.MenuItem("Delete task")
            else:
                del_item = gtk.MenuItem("Delete milestone")
            del_item.connect("activate", self.popup_del_task, task_path )
            task_popup.attach(del_item, 0, 1, 2, 3 )

        task_popup.show_all()
        return task_popup

    def setup_resources_tree(self):
        tree = self.w("treeview_task_resources")
        self.resources_model = gtk.ListStore(gobject.TYPE_STRING, # type
                                             gobject.TYPE_STRING, # id
                                             gobject.TYPE_INT,    # usage
                                             gobject.TYPE_STRING, # color
                                             gobject.TYPE_BOOLEAN, # editable
                                             )
        col = gtk.TreeViewColumn( u"Type", gtk.CellRendererText(), text=0, foreground=3 )
        tree.append_column( col )
        col = gtk.TreeViewColumn( u"ID", gtk.CellRendererText(), text=1, foreground=3 )
        tree.append_column( col )
        col = gtk.TreeViewColumn( u"Usage", gtk.CellRendererText(), text=2, foreground=3 )
        tree.append_column( col )
        tree.set_model( self.resources_model )

    def setup_constraints_tree(self):
        tree = self.w("treeview_task_constraints")
        self.constraints_model = gtk.ListStore(gobject.TYPE_STRING, # type
                                               gobject.TYPE_STRING, # date or res_id
                                               gobject.TYPE_STRING, # color (inherited or not)
                                               gobject.TYPE_BOOLEAN, # editable
                                               )
        rend = gtk.CellRendererCombo()
        rend.set_property('model', self.constraints_type_model )
        rend.set_property('has-entry', False)
        rend.set_property('text-column', 0)
        rend.connect('edited', self.on_constraint_type_edited )
        col = gtk.TreeViewColumn( u"Type", rend, text=0, foreground=2, editable=3 )
        tree.append_column( col )
        rend = gtk.CellRendererCombo()
        rend.set_property('model', self.task_model)
        rend.set_property('text-column', 1)
        rend.connect('edited', self.on_constraint_arg_edited )
        col = gtk.TreeViewColumn( u"Arg", rend, text=1, foreground=2, editable=3 )
        tree.append_column( col )
        tree.set_model( self.constraints_model )
        tree.enable_model_drag_dest( [ ("task", gtk.TARGET_SAME_APP, 0) ],
                                     gtk.gdk.ACTION_COPY )

    def setup_task_tree(self):
        self.task_model = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
        tree = self.w('treeview_all_tasks')
        col = gtk.TreeViewColumn( u"Task", gtk.CellRendererText(), text=1 )
        tree.append_column(col)
        col = gtk.TreeViewColumn( u"Title", gtk.CellRendererText(), text=0 )
        tree.append_column(col)
        tree.set_model( self.task_model )
        sel = tree.get_selection()
        sel.connect("changed", self.on_task_selection_changed )

        # enable drag & drop
        tree.enable_model_drag_dest( [ ("task", gtk.TARGET_SAME_APP, 0) ],
                                     gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_MOVE )
        tree.enable_model_drag_source( gtk.gdk.BUTTON1_MASK,
                                       [ ("task", gtk.TARGET_SAME_APP, 0) ],
                                       gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_MOVE )
        tree.connect("drag-data-received", self.drag_task_received )
        tree.connect("drag-data-get", self.drag_task_get )
        tree.connect("drag-data-delete", self.drag_task_deleted )

    def setup_activities_tree(self):
        self.activities_model = gtk.TreeStore(gobject.TYPE_STRING)
        tree = self.w('treeview_all_activities')
        col = gtk.TreeViewColumn( u"Activities", gtk.CellRendererText(), text=0 )
        tree.append_column(col)
        tree.set_model( self.activities_model )
        sel = tree.get_selection()
        sel.connect("changed", self.on_activities_selection_changed )

    def drag_task_received(self, treeview, context, x, y, selection, info, timestamp):
        print "DRAG RECEIVED:", x, y, info
        print "CTX:", context.action
        root_task = self.app.project.root_task
        task_src_id = selection.data
        task_src = root_task.get_task(task_src_id)
        if not task_src:
            return
        if context.action == gtk.gdk.ACTION_COPY:
            task_new = task_src.copy() # XXX
        elif context.action == gtk.gdk.ACTION_MOVE:
            task_new = task_src

        row = [ task_new.title, task_new.id ]
        drop_info = treeview.get_dest_row_at_pos(x, y)
        parent_id = None  # the parent task id
        sibling_id = None # the closest sibling id or None (append)
        sibling_pos = None # 1: before, 2: after
        if drop_info:
            # drop happened on an item
            model = treeview.get_model()
            path, position = drop_info
            iter = model.get_iter(path)
            sibling_id = model.get_value( iter, 1 )
            parent_itr = model.iter_parent( iter )
            parent_id = model.get_value( parent_itr, 1 )
            parent_dest = root_task.get_task( parent_id )
            if position == gtk.TREE_VIEW_DROP_BEFORE:
                # insert it before the given item
                sibling_pos = 1
                model.insert_before(parent_itr, iter, row )
            elif position == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE:
                # append it after the children of the given item
                if task_src_id == sibling_id:
                    # a task dropped on itself
                    return
                parent_id = sibling_id
                sibling_id = None
                model.append( iter, row )
            else:
                # insert it after the given item
                sibling_pos = 2
                model.insert_after(parent_itr, iter, row)
        else:
            # append it after the children of the root item
            position = None
            parent_dest = root_task
            parent_itr = model.get_iter_first()
            parent_id = model.get_value( parent_itr, 1 )
            sibling_id = None
            model.append( parent_itr, row )

        if context.action == gtk.gdk.ACTION_MOVE:
            task_src.parent.remove( task_src )

        print "DROP", parent_id, task_new.id, position
        if sibling_id is None:
            parent_dest.append( task_new )
        else:
            idx = 0
            for i,c in enumerate(parent_dest.children):
                if c.id == sibling_id:
                    break
            if sibling_pos==2:
                i+=1
            parent_dest.insert(i,task_new)
        context.finish(True, True, timestamp)

    def drag_task_deleted(self, treeview, context):
        print "DRAG DELETED:", context

    def drag_task_get(self, treeview, context, selection, target_id, etime):
        print "DATA GET", target_id
        treeselection = treeview.get_selection()
        model, iter = treeselection.get_selected()
        tid = model.get_value(iter, 1)
        selection.set(selection.target, 8, tid)

    def refresh_task_list(self, sel_task_id=None, sel_task=None):
        model = self.task_model
        model.clear()
        tasks = [ (self.app.project.root_task,None) ]
        while tasks:
            task,parent = tasks.pop(0)
            row = task.title, task.id
            itr = model.append( parent, row )
            for t in task.children:
                tasks.append( (t, itr) )
        tree = self.w('treeview_all_tasks')
        tree.expand_all()
        if sel_task:
            sel_task_id = sel_task.id
        if sel_task_id:
            # select the task
            itr = self.get_task_iter_from_id( sel_task_id )
            tree.get_selection().select_iter( itr )
    
    def get_iter_by_task_activity_id(self, id):
        model = self.activities_model
        itr = model.get_iter_first()
        while itr != None:
            if id == model.get_value(itr,0):
                return itr
            else:
                itr = model.iter_next(itr)
        return itr

    def get_res_iter_from_id_sublevels(self, res_id, itr):
        if itr !=  None:
            rid = self.activities_model.get_value( itr, 0 )
            if res_id == rid:
                return itr
            else:
                return self.get_res_iter_from_id_sublevels(res_id,self.activities_model.iter_next(itr))
        else:
            return None

    def get_iter_by_res_activity_id(self, res_id, itr):
        itr_ = self.get_res_iter_from_id_sublevels(res_id,self.activities_model.iter_children(itr))
        if itr_ != None:
            return itr_
        else:
            return None

    def refresh_activities_list(self, sel_activity_id=None):
        model = self.activities_model
        model.clear()
        ligne=0
        for activity in self.app.project.activities:
            if activity[5]=="past":
                itr = self.get_iter_by_task_activity_id(activity[3])
                if itr is None:
                    itr_ = model.append( itr, [activity[3]])
                    model.append( itr_, [activity[2]])
                else:
                    if self.get_iter_by_res_activity_id(activity[2],itr) != None:
                        model.append( itr, [str( str(activity[2]) + " (" + str(ligne) + ")" )])
                    else:
                        model.append( itr_, [activity[2]])
            ligne = ligne + 1

    def get_task_from_path(self, path):
        root_task = self.app.project.root_task
        itr = self.task_model.get_iter( path )
        if itr:
            task_id = self.task_model.get_value( itr, 1 )
            return root_task.get_task( task_id )
        else:
            return None

    def get_task_iter_from_id_sublevels(self, task_id, itr):
        retour = None
        if itr:
            tid = self.task_model.get_value( itr, 1 )
            if task_id == tid:
                return itr
            else:
                if self.task_model.iter_has_child(itr):
                    retour = self.get_task_iter_from_id_sublevels(task_id,self.task_model.iter_children(itr))
                    if retour != None:
                        return retour
                retour = self.get_task_iter_from_id_sublevels(task_id,self.task_model.iter_next(itr))
                if retour != None:
                    return retour

    def get_task_iter_from_id(self, task_id):
        itr = self.task_model.get_iter_first()
        itr_ = self.get_task_iter_from_id_sublevels(task_id,itr)
        if itr_ != None:
            return itr_
        else:
            return itr

    def on_project_changed(self, app):
        """Propagates the fact that the project file
        has changed"""
        print app.project
        print app.files
        self.setup_project_files_path()
        self.refresh_task_list()
        self.refresh_activities_list()
        

    def on_task_selection_changed(self, sel):
        model, itr = sel.get_selected()
        if not itr:
            return
        task_id = model.get_value(itr, 1)
        self.current_task = self.app.project.root_task.get_task(task_id)
        self.current_task_path = model.get_path( itr )
        self.update_task_info()

    def on_activities_selection_changed(self, sel):
        model, itr = sel.get_selected()
        if not itr:
            return
        self.current_activity_path = model.get_path( itr )
        self.current_activity_itr = itr
        self.update_activities_info()

    def update_task_info(self):
        task = self.current_task
        # update widgets value
        self.w("entry_task_id").set_text( task.id )
        self.w("entry_task_title").set_text( task.title )
        #buf = .get_buffer()
        buf = gtksourceview2.Buffer()
        buf.connect("changed", self.on_textview_task_description_changed )
        self.w("textview_task_description").set_buffer( buf )
        if task.description_format == "rest":
            buf.set_language(None)
            self.w("combobox_description_format").set_active(1)
        elif task.description_format == "docbook":
            buf.set_language( XMLLANG )
            self.w("combobox_description_format").set_active(0)
        else:
            self.w("combobox_description_format").set_active(2)

        #securité acces load si taches filles
        has_child = task.children
        if has_child:
            task.duration=0.0
        self.w("spinbutton_duration").set_sensitive(not has_child)
        self.w("combobox_scheduling_type").set_sensitive(not has_child)

        if task.TYPE=="milestone":
            self.w("combobox_scheduling_type").set_sensitive(False)
            self.w("spinbutton_duration").set_sensitive(False)

        buf.set_text( task.description_raw )
        self.w("spinbutton_duration").get_adjustment().set_value( task.duration )
        self.w("combobox_scheduling_type").set_active( task.load_type )

        self.constraints_model.clear()
        child = task
        color = "black"
        while child:
            for constraint_type, arg in child.task_constraints:
                self.constraints_model.append( (constraint_type, arg, color, color=='black' ) )
            child = child.parent
            color = "gray"

        if isinstance(task, Task):
            child = task
        else:
            # Milestones don't have resources
            child = None 
        color = "black"
        while child and not child.resource_constraints:
            child = child.parent
            color = "gray"
        
        self.resources_model.clear()
        if child:
            for res_type, res_id, res_usage in child.resource_constraints:
                self.resources_model.append( (res_type, res_id, res_usage, color, color=='black') )

    def get_activity_by_res_task_id(self, rid, tid):
        for activity in self.app.project.activities:
            if activity[2] == rid and activity[3] == tid:
                return activity
        return None    

    def get_activity_by_line_id(self, line_id):
        ligne = 0
        for activity in self.app.project.activities:
            if str(ligne) == str(line_id):
                return activity
            ligne = ligne + 1
        return None

    def update_activities_info(self):
        model = self.activities_model
        type_activity = model.iter_depth(model.get_iter(self.current_activity_path))

        if type_activity == 0:
            self.w("entry_activities_id").set_text(model.get_value(self.current_activity_itr,0))
            self.w("entry_activities_from").set_sensitive(False)
            self.w("entry_activities_to").set_sensitive(False)
            self.w("spinbutton_activities_usage").set_sensitive(False)
            self.w("combobox_activities_resource").set_sensitive(False)
            self.w("entry_activities_from").set_text("")
            self.w("entry_activities_to").set_text("")
            self.w("spinbutton_activities_usage").set_value(0)

        if type_activity == 1:
            res_id = model.get_value(self.current_activity_itr,0)
            if res_id.find("(")>0:
                line = res_id[res_id.find("(")+1:res_id.find(")")]
                res_id = res_id[0:res_id.find("(")]
                activity = self.get_activity_by_line_id(line)
            else:
                activity = self.get_activity_by_res_task_id(res_id,self.activities_model.get_value(self.activities_model.iter_parent(self.current_activity_itr),0))
            self.w("entry_activities_id").set_text(res_id)
            self.w("entry_activities_from").set_sensitive(True)
            self.w("entry_activities_to").set_sensitive(True)
            self.w("spinbutton_activities_usage").set_sensitive(True)
            self.w("combobox_activities_resource").set_sensitive(False)
            self.w("entry_activities_from").set_text(str(activity[0].date))
            self.w("entry_activities_to").set_text(str(activity[1].date))
            self.w("spinbutton_activities_usage").set_value(int(activity[4]))

    def on_entry_task_title_changed(self, entry):
        itr = self.task_model.get_iter( self.current_task_path )
        title = entry.get_text()
        self.current_task.title = title
        self.task_model.set_value( itr, 0, title )

    def on_entry_task_id_changed(self, entry):
        itr = self.task_model.get_iter( self.current_task_path )
        task_id = entry.get_text()
        self.current_task.id = task_id
        self.task_model.set_value( itr, 1, task_id )
        
    def on_spinbutton_duration_changed(self, spin):
        dur = self.w("spinbutton_duration").get_adjustment().get_value()
        self.current_task.duration = dur

    def on_combobox_scheduling_type_changed(self, combo):
        model = combo.get_model()
        iter = combo.get_active_iter()
        load_type = model.get_value(iter, 0)
        self.current_task.load_type = LOAD_TYPE_MAP[load_type]
        self.update_task_info()

    def on_combobox_description_format_changed(self, combo):
        model = combo.get_model()
        iter = combo.get_active_iter()
        description_format = model.get_value(iter, 0)
        self.current_task.description_format = description_format.lower()

    def on_treeview_all_tasks_button_press_event(self, treeview, event):
        if event.button != 3:
            return None
        x = int(event.x)
        y = int(event.y)
        time = event.time
        pthinfo = treeview.get_path_at_pos(x, y)
        if pthinfo is not None:
            #print pthinfo
            path, col, cellx, celly = pthinfo
            treeview.grab_focus()
            treeview.set_cursor( path, col, 0)
        else:
            path = None
        if self.task_popup:
            self.task_popup.destroy()
        self.task_popup = self.build_task_tree_popup(path)
        self.task_popup.popup( None, None, None, event.button, time)
        return 1

    def popup_add_task(self, item, path):
        if path is None:
            return
        root_task = self.app.project.root_task
        parent_task = self.get_task_from_path( path )

        if not parent_task.children:
            dlg = gtk.MessageDialog(parent=None, flags=0, 
                    type=gtk.MESSAGE_QUESTION, 
                    buttons=gtk.BUTTONS_YES_NO, 
                    message_format= "Warning : Adding a Task as a child of Task(%s) will reset the load value of it. \n Are you sure ?" % parent_task.id);
            ret = dlg.run()
            dlg.destroy()
            if ret == gtk.RESPONSE_NO:
                return

        if not parent_task:
            parent_task = root_task
        if not isinstance(parent_task, Task):
            print "XXX: CAN'T ADD TASK TO MILESTONE"
            return
        ids = set([ task.id for task in root_task.flatten() ])
        i = 1
        while 1:
            new_id = parent_task.id+"_%s"%i
            if new_id not in ids:
                break
            i = i + 1
        new_task = MODEL_FACTORY.create_task( new_id )
        new_task.title = "#TODO"
        new_task.description_raw = "#TODO"
        parent_task.append( new_task )
        self.refresh_task_list(sel_task_id=new_id)

    def popup_add_milestone(self, item, path):
        if path is None:
            return
        root_task = self.app.project.root_task
        parent_task = self.get_task_from_path( path )

        if not parent_task.children:
            dlg = gtk.MessageDialog(parent=None, flags=0, 
                    type=gtk.MESSAGE_QUESTION, 
                    buttons=gtk.BUTTONS_YES_NO, 
                    message_format= "Warning : Adding a Milestone as a child of Task(%s) will reset the load value of it. \n Are you sure ?" % parent_task.id);
            ret = dlg.run()
            dlg.destroy()
            if ret == gtk.RESPONSE_NO:
                return

        if not parent_task:
            parent_task = root_task
        if not isinstance(parent_task, Task):
            print "XXX: CAN'T MILESTONE TASK TO MILESTONE"
            return
        ids = set([ task.id for task in root_task.flatten() ])
        i = 1
        while 1:
            new_id = "milestone_"+parent_task.id+"_%s"%i
            if new_id not in ids:
                break
            i = i + 1
        new_task = MODEL_FACTORY.create_milestone( new_id )
        new_task.TYPE="milestone"
        new_task.load_type=LOAD_TYPE_MAP["milestone"]
        new_task.title = "#TODO"
        parent_task.append( new_task )
        self.refresh_task_list(sel_task_id=new_id)

    def popup_del_task(self, item, path):
        task = self.get_task_from_path( path )
        parent = task.parent
        if not parent:
            dlg = gtk.MessageDialog(parent=None, flags=0, type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_OK, message_format= "Error : Root task can't be deleted.")
            dlg.run()
            dlg.destroy()
            return
        if task.children:
            dlg = gtk.MessageDialog(parent=None, flags=0, type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_OK, message_format= "Error : Can't delete task(%s), task has children." % task.id)
            dlg.run()
            dlg.destroy()
            return
        parent.remove( task )
        self.refresh_task_list(sel_task=parent)

    def on_textview_task_description_changed(self, buf):
        _beg = buf.get_start_iter()
        _end = buf.get_end_iter()
        txt = buf.get_text( _beg, _end )
        self.current_task.description_raw = txt

    def on_constraint_type_edited(self, renderer, path, new_text):
        itr = self.constraints_model.get_iter( path )
        constr, value = self.constraints_model.get( itr, 0, 1 )
        assert constr in TASK_CONSTRAINTS
        self.current_task.task_constraints.remove( (constr, value) )
        self.current_task.add_task_constraint( new_text, value )
        self.constraints_model.set_value( itr, 0, new_text )

    def on_constraint_arg_edited(self, renderer, path, new_text):
        itr = self.constraints_model.get_iter( path )
        constr, value = self.constraints_model.get( itr, 0, 1 )
        self.current_task.task_constraints.remove( (constr, value) )
        self.current_task.add_task_constraint( constr, new_text )
        self.constraints_model.set_value( itr, 1, new_text )

    def on_button_project_resources_show_button_press_event(self, button, evt):
        self.w("notebook1").set_current_page(1)

    def on_button_project_activities_show_button_press_event(self, button, evt):
        self.w("notebook1").set_current_page(2)

    def on_button_project_tasks_show_button_press_event(self, button, evt):
        self.w("notebook1").set_current_page(3)

    def on_button_project_schedule_show_button_press_event(self, button, evt):
        self.w("notebook1").set_current_page(4)

    def on_notebook1_switch_page(self, notebook, page, page_index):
        if notebook.get_tab_label(notebook.get_nth_page(page_index)).get_text() == "Tasks":
            if self.current_task is None:
                self.current_task = self.app.project.root_task
            self.refresh_task_list(sel_task=self.current_task)
            self.update_task_info()




# -*- coding: utf-8 -*-
# Copyright (c) 2000-2006 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from projman.scheduling import schedule
from projman.scheduling.csp import CSPScheduler

class Checker:
    
    def __init__(self, project, verbose=0, **kw):
        scheduler = CSPScheduler(project)
        self.project=project
        self.verbosity = verbose
        self.real_tasks = scheduler.real_tasks
        self.constraints = scheduler.constraints
        self.resources = scheduler.resources
        self.max_duration = scheduler.max_duration
        self.successors = {}
        self.predecessors = {}
        self.first_day = None
        self.trees = None
        self.errors = []

    def problem_checker(self):
        if self.verbosity:
            self.dump()
        self.validate()
        self.check_tasks_convexity()
        self.check_tree()
        if self.trees:
            self.check_first_days()
        if self.errors:
            print 'attention, malformed problem:'
        else:
            print 'Problem valid'
        for error in self.errors:
            print '* ', error

    def dump(self):
        """ print principal data of the probleme """
        print "\n#  Set of real tasks (no container) #"
        for task_id in self.real_tasks:
            task = self.project.get_task(task_id)
            print task.title, '(id =', task.id, '; duration = ',task.duration,')' 
        print "\n#  Avaliable resources for the project #"
        for res in self.resources:
            res = self.project.get_resource(res)
            print res.name, " (",res.id_role,")"
        print "\n#  Set of constraints #"
        for task in self.project.root_task.leaves():
            for c_type, date, priority in task.get_date_constraints():
                print task.id, c_type, date, "(priority",priority,")"
        for constraint in self.constraints:
            set_tasks = self.constraints.get(constraint)
            print constraint,":"
            for couple in set_tasks:
                print couple[0], constraint, couple[1]
        print'\n'

    def validate(self):
        """check that there is no duplicate id or null duration of leaves"""
        consistence_errors = self.project.check_consistency()
        if consistence_errors:
            for error in consistence_errors:
                self.errors.append(error)
        # check tasks duration for leaves
        for leaf in self.project.root_task.leaves():
            if leaf.duration == 0 and leaf.TYPE=='task':
                self.errors.append('Leaf %s without any duration:\n\t-> add duration, add sub tasks or write it as a milestone' %leaf.id)

    def check_tree(self):
        tasks = []
        for tid in self.real_tasks:
            tasks.append(tid)
        arcs = []
        for constraint in self.constraints:
            if constraint in ('begin-after-end', 'begin-after-begin'):
                for couple in self.constraints.get(constraint):
                    arcs.append((couple[1], couple[0]))
        for task in tasks:
            self.predecessors.setdefault(task, [])
            self.successors.setdefault(task, [])
        for couple in arcs:
            self.predecessors[couple[1]].append(couple[0])
            self.successors[couple[0]].append(couple[1])
            
        if self.verbosity > 1:
            print "arcs", arcs
            print "tasks", tasks
            print "listes des predecesseurs",self.predecessors
            print "liste des successeurs   ", self.successors
        self.trees = self.depth_first_search(tasks)

    def depth_first_search(self, tasks):
        """ applie DFS algorithm in set of tasks to detect cycle and find
            tasks order """
        if self.verbosity > 1:
            print 'ALGORITHME DEPTH FIRST SEARCH'
        if not [] in self.predecessors.values() or not [] in self.successors.values():
            self.errors.append('Cycles detected:\n\t-> change tasks constraints')
            return []
        trees = []
        set_tag = set()
        tag = None
        if self.verbosity > 2:
            print "predecessors:", self.predecessors
            print "successors:", self.successors

        for key in self.predecessors:
            # detect fisrt task
            tag = []            
            values = self.predecessors.get(key)
            if values == []:
                first = key
            else:
                continue
            Q = []
            Q.append(first)
            rounder = 0
            while Q:
                rounder += 1
                if self.verbosity > 2:
                    print "\nQ"  , Q
                    print 'tag', tag
                on_good_path = True
                v = Q[0]
                if v == first and rounder > 1:
                     self.errors.append('Cycles detected:\n\t-> change tasks constraints')
                     return []
                #check if all predecessors are allready in the path
                for pred in self.predecessors.get(v):
                    if not pred in tag and not pred in Q:
                        if pred in Q:
                            Q.remove(pred)
                        Q.insert(0, pred)
                        on_good_path = False
                if not on_good_path:
                    continue
                Q.remove(v)
                tag.append(v)
                for v_prime in self.successors.get(v):
                    if not v_prime in tag:
                        if self.verbosity > 2:
                            print '  successors:', v_prime
                        if not v_prime in Q:
                            Q.append(v_prime)                    
            trees.append(tag)
            for elt in tag:
                set_tag.add(elt)
            # check if every real task was considered
        for task in self.real_tasks:
            if not task in set_tag:
                self.errors.append('Cycles detected on task %s:\n\t-> change tasks constraints' %task)
                return []
        if self.verbosity:
            print 'tasks paths'
            print trees
        return trees

    def check_first_days(self):
        """detect unconsistant probleme due to the hypothese that
        the fist day of a project must be work """
        first = []
        for task in self.project.root_task.leaves():
            for c_type, date, priority in task.get_date_constraints():
                if c_type in ('begin-after-date', 'begin-at-date'):
                    first.append(date)
        self.first_day = min(first)
        for tree in self.trees:
            # find date constraints on first task of the tree
            task = self.project.get_task(tree[0])
            for c_type, date, priority in task.get_date_constraints():
                if date > self.first_day:
                    self.errors.append('Incoherent date constraint in task %s:\n\t-> change date constraints' % task.id)

    def check_tasks_convexity(self):
        for leaf_id in self.real_tasks:
            leaf = self.project.get_task(leaf_id)
            # compute the number of possible resources for the task
            # according to new definition of resources role
            if not leaf.can_interrupt[0] and leaf.duration >5:
                count = 0
                for res_id in self.resources:
                    resource = self.project.get_resource(res_id)
                    if leaf.task_type in resource.id_role:
                        count += 1
                duration = leaf.duration / float(count)
                if duration > 5:
                    self.errors.append('Uninterruptible task %s with duration in excess of one week:\n\t-> reduce duration or associate the task to more resources' %leaf.id)
                



# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=W0622
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""Copyright (c) 2000-2008 LOGILAB S.A. (Paris, FRANCE).
http://www.logilab.fr/ -- mailto:contact@logilab.fr  
"""

modname = 'projman'

numversion = (0, 13, 5)
version = '.'.join([str(num) for num in numversion])


license = 'GPL'
copyright = '''Copyright © 2000-2008 LOGILAB S.A. (Paris, FRANCE).
http://www.logilab.fr/ -- mailto:contact@logilab.fr'''

short_desc = "tool for project management"

long_desc = """projman is a tool for project management and diagrams creation,
as Gantt diagram and resources activities diagram.
It includes lots of functionnalties as
* scheduling of a project
* managing work-groups'actvities
* Gantt diagrams generation
* Resources'usage diagrams generation
* generation of xml doc (under docbook dtd) to list tasks, evaluate costs

All different format and output are XML files.
"""

author = "Logilab"
author_email = "devel@logilab.fr"

web = "http://www.logilab.org/projects/%s" % modname
ftp = "ftp://ftp.logilab.org/pub/%s" % modname
mailinglist = "http://lists.logilab.org/mailman/listinfo/management-projects"


scripts = ['bin/projman', ]
data_files = [['share/projman',
               ['fonts/Arial_12_72.pil',
                'fonts/Arial Bold_12_72.pil',
                'fonts/Arial Italic_12_72.pil',
                'fonts/Arial Bold Italic_12_72.pil',
                'fonts/Arial_12_72.pbm',
                'fonts/Arial Bold_12_72.pbm',
                'fonts/Arial Italic_12_72.pbm',
                'fonts/Arial Bold Italic_12_72.pbm'],
               ['share/projman/examples',
                ['scheduling/sample.cc',
                 'scheduling/projman_gecode.cc',
                 'scheduling/makefile',
                 'scheduling/projman_gecode.hh',
                 'scheduling/projman_problem.hh',
                 'scheduling/timer.hh',
                ]
               ] 
              ]
             ]

debian_name = 'projman'
debian_maintainer = 'Alexandre Fayolle ' 
debian_maintainer_email = 'alexandre.fayolle@logilab.fr'
pyversions = ["2.5"]
 
from os.path import join
include_dirs = [join('test', 'data')]

from distutils.core import Extension
ext_modules = [Extension('projman.scheduling.gcsp',
                         sources = ['scheduling/gcspmodule.cc',
                                    'scheduling/projman_gecode.cc',
                                    'scheduling/projman_problem.cc',
                         ],
                         libraries=['boost_python-mt', 'gecodeint', 'gecodeset',
                                    'gecodeminimodel', 'gecodekernel', 'gecodesearch'],
                         depends=['scheduling/projman_gecode.hh',
                                  'scheduling/projman_problem.hh',
                                  ],
                         language='c++',
                        )
             ]





# Copyright (c) 2000-2003 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
""" base classes for rendering """

__revision__ = "$Id: ganttresource.py,v 1.1 2005-09-06 17:07:24 nico Exp $"

from projman.renderers.resource import ResourcesRenderer, ResourcesDrawer
from projman.renderers.gantt import GanttRenderer, GanttDrawer
from projman.renderers.abstract import AbstractRenderer, ROW_HEIGHT

class GanttResourcesRenderer(ResourcesRenderer, GanttRenderer):
    
    def __init__(self, options, handler, colors_file=None, colors_stream=None):
        AbstractRenderer.__init__(self, options)
        self.drawer = GanttResourcesDrawer(options, handler, colors_file,
                                           colors_stream)
        self._draw_resources = {}
        
    def render(self, project, stream):
        """
        render the task as a gantt & resources diagram
        """
        AbstractRenderer.render(self, project, stream)
        self.drawer._handler.save_result(stream)
       
    def _render_body(self, project) :
        GanttRenderer._render_body(self, project)
        ResourcesRenderer._render_body(self, project)

    def render_resource(self, resource, project) :
        """
        generate events for a given resource
        """
        self.drawer.set_color_set(self._i)
        self._i += 1
        self.render_total_occupation(project, resource)

class GanttResourcesDrawer(ResourcesDrawer, GanttDrawer):
    def _init_table(self):
        """ initialize fields needed by the table"""
        GanttDrawer._init_table(self) 
        ResourcesDrawer._init_table(self)

    def _get_table_dimension(self, project):
        """ calculate dimension of the table"""
        #retreive attributes from mother classes
        width_gantt, height_gantt = GanttDrawer._get_table_dimension(self, project) 
        width_res, height_res = ResourcesDrawer._get_table_dimension(self, project)
        #calculate height_resource_reduced
        nb_resources = len(project.resource_set.get_resources())
        nb_tasks = len(project.root_task.flatten())
        height_resource_reduced = ROW_HEIGHT*(5+nb_tasks+nb_resources*2)
        width = max(width_gantt, width_res)
        height = height_gantt+height_resource_reduced
        return width, height
                    
